@@ -7,11 +7,13 @@ from datetime import datetime
 from cStringIO import StringIO
 from ConfigParser import ConfigParser
 from itertools import islice
+import tarfile
+from shutil import rmtree
 
 import tg
 from pylons import app_globals as g
 os.environ['HGRCPATH'] = '' # disable loading .hgrc
-from mercurial import ui, hg, cmdutil
+from mercurial import ui, hg, cmdutil, commands
 
 from pymongo.errors import DuplicateKeyError
 
@@ -321,4 +323,25 @@ class HgImplementation(M.RepositoryImplementation):
     def commits_count(self, path=None, rev=None):
         return sum(1 for _ in self.commits(path=path, rev=rev))
 
+    def tarball(self, commit):
+        shortname = self._repo.project.shortname.replace('/', '-')
+        mount_point = self._repo.app.config.options.mount_point
+        if not os.path.exists(self._repo.tarball_path):
+            os.makedirs(self._repo.tarball_path)
+        archive_name = '%s-%s-%s' % (shortname, mount_point, commit)
+        path = os.path.join(self._repo.tarball_path, archive_name)
+        archive_file = path + '.tar.gz'
+        if os.path.exists(path):
+            rmtree(path)
+        commands.archive(
+            HgUI(),
+            self._hg,
+            path,
+            rev=commit,
+            prefix='')
+        with tarfile.open(archive_file, 'w:gz') as tar:
+            tar.add(path, arcname=archive_name)
+        rmtree(path)
+
+
 Mapper.compile_all()
